import { Version } from '#bin/lib/types';
import { FunctionParser } from '#lib/structures/FunctionParser';
import { NamespaceParser } from '#lib/structures/NamespaceParser';
import { ProjectParser } from '#lib/structures/ProjectParser';
import { TypeAliasParser } from '#lib/structures/TypeAliasParser';
import { VariableParser } from '#lib/structures/VariableParser';
import { ClassConstructorParser, ClassMethodParser, ClassParser, ClassPropertyParser } from '#lib/structures/class-parser';
import { EnumMemberParser, EnumParser } from '#lib/structures/enum-parser';
import { InterfaceMethodParser, InterfaceParser, InterfacePropertyParser } from '#lib/structures/interface-parser';
import { CommentParser, MethodParser, ParameterParser, PropertyParser, SignatureParser, SourceParser, TypeParameterParser } from '#lib/structures/misc';
import { TypeParser } from '#lib/structures/type-parsers';
import { ReflectionKind } from '#lib/types';
import { bold, yellow } from 'colorette';
const currentTypeDocJsonParserVersion = ProjectParser.version
    .split('.')
    // eslint-disable-next-line radix
    .map((x) => parseInt(x))
    .slice(0, 3)
    .join('.');
export function migrateProjectJson(projectJson) {
    const typeDocJsonParserVersion = 'typeDocJsonParserVersion' in projectJson ? projectJson.typeDocJsonParserVersion : '2.0.0';
    if (typeDocJsonParserVersion === currentTypeDocJsonParserVersion) {
        return projectJson;
    }
    // eslint-disable-next-line radix
    const [currentMajor, currentMinor, currentPatch] = currentTypeDocJsonParserVersion.split('.').map((x) => parseInt(x));
    // eslint-disable-next-line radix
    const [major, minor, patch] = typeDocJsonParserVersion.split('.').map((x) => parseInt(x));
    if (major > currentMajor ||
        (major === currentMajor && minor > currentMinor) ||
        (major === currentMajor && minor === currentMinor && patch > currentPatch)) {
        return yellow(`${bold('[WARN]')} Unknown version of "typedoc-json-parser" encountered. Received "${typeDocJsonParserVersion}" from "${projectJson.name}"${'version' in projectJson ? ` with version "${projectJson.version}"` : ''}.`);
    }
    const { id, name, classes, enums, functions, interfaces, namespaces, typeAliases } = projectJson;
    if (major < 6) {
        const { constants } = projectJson;
        return {
            typeDocJsonParserVersion: currentTypeDocJsonParserVersion,
            id,
            name,
            version: 'version' in projectJson ? projectJson.version : null,
            dependencies: {},
            readme: 'readme' in projectJson ? projectJson.readme : null,
            changelog: 'changelog' in projectJson ? projectJson.changelog : null,
            classes: classes.map((classJson) => migrateClassJson(classJson, null, [major, minor, patch])),
            enums: enums.map((enumJson) => migrateEnumJson(enumJson, null, [major, minor, patch])),
            functions: functions.map((functionJson) => migrateFunctionJson(functionJson, null, [major, minor, patch])),
            interfaces: interfaces.map((interfaceJson) => migrateInterfaceJson(interfaceJson, null, [major, minor, patch])),
            namespaces: namespaces.map((namespaceJson) => migrateNamespaceJson(namespaceJson, null, [major, minor, patch])),
            typeAliases: typeAliases.map((typeAliasJson) => migrateTypeAliasJson(typeAliasJson, null, [major, minor, patch])),
            variables: constants.map((constantJson) => migrateVariableJson(constantJson, null, [major, minor, patch]))
        };
    }
    const { version, readme, changelog, variables } = projectJson;
    return {
        typeDocJsonParserVersion: currentTypeDocJsonParserVersion,
        id,
        name,
        version,
        dependencies: 'dependencies' in projectJson ? projectJson.dependencies : {},
        readme,
        changelog,
        classes: classes.map((classJson) => migrateClassJson(classJson, null, [major, minor, patch])),
        enums: enums.map((enumJson) => migrateEnumJson(enumJson, null, [major, minor, patch])),
        functions: functions.map((functionJson) => migrateFunctionJson(functionJson, null, [major, minor, patch])),
        interfaces: interfaces.map((interfaceJson) => migrateInterfaceJson(interfaceJson, null, [major, minor, patch])),
        namespaces: namespaces.map((namespaceJson) => migrateNamespaceJson(namespaceJson, null, [major, minor, patch])),
        typeAliases: typeAliases.map((typeAliasJson) => migrateTypeAliasJson(typeAliasJson, null, [major, minor, patch])),
        variables: variables.map((variableJson) => migrateVariableJson(variableJson, null, [major, minor, patch]))
    };
}
function migrateClassJson(classJson, namespaceParentId, typeDocJsonParserVersion) {
    const [major] = typeDocJsonParserVersion;
    const { id, name, comment, source, external, abstract, extendsType, implementsType, construct, properties, methods } = classJson;
    if (major < 6) {
        return {
            id,
            name,
            namespaceParentId,
            comment: migrateCommentJson(comment, typeDocJsonParserVersion),
            source: source ? migrateSourceJson(source, typeDocJsonParserVersion) : null,
            external,
            abstract,
            extendsType: extendsType ? migrateTypeJson(extendsType, typeDocJsonParserVersion) : null,
            implementsType: implementsType ? implementsType.map((typeJson) => migrateTypeJson(typeJson, typeDocJsonParserVersion)) : [],
            typeParameters: [],
            construct: migrateClassConstructorJson(construct, id, typeDocJsonParserVersion),
            properties: properties.map((propertyJson) => migrateClassPropertyJson(propertyJson, id, typeDocJsonParserVersion)),
            methods: methods.map((methodJson) => migrateClassMethodJson(methodJson, id, typeDocJsonParserVersion))
        };
    }
    const { typeParameters } = classJson;
    return {
        id,
        name,
        namespaceParentId,
        comment: migrateCommentJson(comment, typeDocJsonParserVersion),
        source: source ? migrateSourceJson(source, typeDocJsonParserVersion) : null,
        external,
        abstract,
        extendsType: extendsType ? migrateTypeJson(extendsType, typeDocJsonParserVersion) : null,
        implementsType: implementsType ? implementsType.map((typeJson) => migrateTypeJson(typeJson, typeDocJsonParserVersion)) : [],
        typeParameters: typeParameters
            ? typeParameters.map((typeParameterJson) => migrateTypeParameterJson(typeParameterJson, typeDocJsonParserVersion))
            : [],
        construct: migrateClassConstructorJson(construct, id, typeDocJsonParserVersion),
        properties: properties.map((propertyJson) => migrateClassPropertyJson(propertyJson, id, typeDocJsonParserVersion)),
        methods: methods.map((methodJson) => migrateClassMethodJson(methodJson, id, typeDocJsonParserVersion))
    };
}
function migrateClassConstructorJson(constructorJson, parentId, typeDocJsonParserVersion) {
    const [major] = typeDocJsonParserVersion;
    const { id, name, comment, source, parameters } = constructorJson;
    if (major < 7) {
        return {
            id,
            name,
            comment: migrateCommentJson(comment, typeDocJsonParserVersion),
            parentId,
            source: source ? migrateSourceJson(source, typeDocJsonParserVersion) : null,
            accessibility: ClassParser.Accessibility.Public,
            parameters: parameters.map((parameterJson) => migrateParameterJson(parameterJson, typeDocJsonParserVersion))
        };
    }
    const { accessibility } = constructorJson;
    return {
        id,
        name,
        comment: migrateCommentJson(comment, typeDocJsonParserVersion),
        parentId,
        source: source ? migrateSourceJson(source, typeDocJsonParserVersion) : null,
        accessibility,
        parameters: parameters.map((parameterJson) => migrateParameterJson(parameterJson, typeDocJsonParserVersion))
    };
}
function migrateClassPropertyJson(propertyJson, parentId, typeDocJsonParserVersion) {
    const { id, name, comment, source, accessibility, abstract, static: _static, readonly, optional, type } = propertyJson;
    return {
        id,
        name,
        comment: migrateCommentJson(comment, typeDocJsonParserVersion),
        source: source ? migrateSourceJson(source, typeDocJsonParserVersion) : null,
        parentId,
        accessibility,
        abstract,
        static: _static,
        readonly,
        optional,
        type: migrateTypeJson(type, typeDocJsonParserVersion)
    };
}
function migrateClassMethodJson(methodJson, parentId, typeDocJsonParserVersion) {
    const { id, name, source, accessibility, abstract, static: _static, signatures } = methodJson;
    return {
        id,
        name,
        source: source ? migrateSourceJson(source, typeDocJsonParserVersion) : null,
        parentId,
        accessibility,
        abstract,
        static: _static,
        signatures: signatures.map((signatureJson) => migrateSignatureJson(signatureJson, typeDocJsonParserVersion))
    };
}
function migrateEnumJson(enumJson, namespaceParentId, typeDocJsonParserVersion) {
    const [major] = typeDocJsonParserVersion;
    const { id, name, comment, source, external } = enumJson;
    if (major < 6) {
        const { properties } = enumJson;
        return {
            id,
            name,
            comment: migrateCommentJson(comment, typeDocJsonParserVersion),
            source: source ? migrateSourceJson(source, typeDocJsonParserVersion) : null,
            namespaceParentId,
            external,
            members: properties.map((propertyJson) => migrateEnumMemberJson(propertyJson, id, typeDocJsonParserVersion))
        };
    }
    const { members } = enumJson;
    return {
        id,
        name,
        comment: migrateCommentJson(comment, typeDocJsonParserVersion),
        source: source ? migrateSourceJson(source, typeDocJsonParserVersion) : null,
        namespaceParentId,
        external,
        members: members.map((memberJson) => migrateEnumMemberJson(memberJson, id, typeDocJsonParserVersion))
    };
}
function migrateEnumMemberJson(enumMemberJson, parentId, typeDocJsonParserVersion) {
    const { id, name, comment, source, value } = enumMemberJson;
    return {
        id,
        name,
        comment: migrateCommentJson(comment, typeDocJsonParserVersion),
        source: source ? migrateSourceJson(source, typeDocJsonParserVersion) : null,
        parentId,
        value
    };
}
function migrateInterfaceJson(interfaceJson, namespaceParentId, typeDocJsonParserVersion) {
    const [major, minor] = typeDocJsonParserVersion;
    const { id, name, comment, source, external, properties } = interfaceJson;
    if (major < 4) {
        if (!(major === 3 && minor >= 1)) {
            return {
                id,
                name,
                comment: migrateCommentJson(comment, typeDocJsonParserVersion),
                source: source ? migrateSourceJson(source, typeDocJsonParserVersion) : null,
                namespaceParentId,
                external,
                typeParameters: [],
                properties: properties.map((propertyJson) => migrateInterfacePropertyJson(propertyJson, id, typeDocJsonParserVersion)),
                methods: []
            };
        }
    }
    if (major < 7) {
        if (!(major === 3 && minor <= 1)) {
            const { methods } = interfaceJson;
            return {
                id,
                name,
                comment: migrateCommentJson(comment, typeDocJsonParserVersion),
                source: source ? migrateSourceJson(source, typeDocJsonParserVersion) : null,
                namespaceParentId,
                external,
                typeParameters: [],
                properties: properties.map((propertyJson) => migrateInterfacePropertyJson(propertyJson, id, typeDocJsonParserVersion)),
                methods: methods.map((methodJson) => migrateInterfaceMethodJson(methodJson, id, typeDocJsonParserVersion))
            };
        }
    }
    const { typeParameters, methods } = interfaceJson;
    return {
        id,
        name,
        comment: migrateCommentJson(comment, typeDocJsonParserVersion),
        source: source ? migrateSourceJson(source, typeDocJsonParserVersion) : null,
        namespaceParentId,
        external,
        typeParameters: typeParameters.map((typeParameterJson) => migrateTypeParameterJson(typeParameterJson, typeDocJsonParserVersion)),
        properties: properties.map((propertyJson) => migrateInterfacePropertyJson(propertyJson, id, typeDocJsonParserVersion)),
        methods: methods.map((methodJson) => migrateInterfaceMethodJson(methodJson, id, typeDocJsonParserVersion))
    };
}
function migrateInterfacePropertyJson(propertyJson, parentId, typeDocJsonParserVersion) {
    const [major, minor] = typeDocJsonParserVersion;
    const { id, name, comment, source, readonly, type } = propertyJson;
    if (major < 9) {
        if (major === 8 && minor < 2) {
            return {
                id,
                name,
                comment: migrateCommentJson(comment, typeDocJsonParserVersion),
                source: source ? migrateSourceJson(source, typeDocJsonParserVersion) : null,
                parentId,
                readonly,
                optional: false,
                type: migrateTypeJson(type, typeDocJsonParserVersion)
            };
        }
    }
    const { optional } = propertyJson;
    return {
        id,
        name,
        comment: migrateCommentJson(comment, typeDocJsonParserVersion),
        source: source ? migrateSourceJson(source, typeDocJsonParserVersion) : null,
        parentId,
        readonly,
        optional,
        type: migrateTypeJson(type, typeDocJsonParserVersion)
    };
}
function migrateInterfaceMethodJson(methodJson, parentId, typeDocJsonParserVersion) {
    const { id, name, source, signatures } = methodJson;
    return {
        id,
        name,
        source: source ? migrateSourceJson(source, typeDocJsonParserVersion) : null,
        parentId,
        signatures: signatures.map((signatureJson) => migrateSignatureJson(signatureJson, typeDocJsonParserVersion))
    };
}
function migrateCommentJson(commentJson, _typeDocJsonParserVersion) {
    return commentJson;
}
function migrateMethodJson(methodJson, _typeDocJsonParserVersion) {
    return methodJson;
}
function migrateParameterJson(parameterJson, typeDocJsonParserVersion) {
    const [major, minor] = typeDocJsonParserVersion;
    const { id, name, type } = parameterJson;
    if (major < 6) {
        return {
            id,
            name,
            comment: { description: null, blockTags: [], modifierTags: [] },
            rest: false,
            optional: false,
            type: migrateTypeJson(type, typeDocJsonParserVersion)
        };
    }
    if (major < 8) {
        if (!(major === 7 && minor >= 1)) {
            const { comment } = parameterJson;
            return {
                id,
                name,
                comment: migrateCommentJson(comment, typeDocJsonParserVersion),
                rest: false,
                optional: false,
                type: migrateTypeJson(type, typeDocJsonParserVersion)
            };
        }
        if (major === 7 && minor === 1) {
            const { comment, optional } = parameterJson;
            return {
                id,
                name,
                comment: migrateCommentJson(comment, typeDocJsonParserVersion),
                rest: false,
                optional,
                type: migrateTypeJson(type, typeDocJsonParserVersion)
            };
        }
    }
    const { comment, optional, rest } = parameterJson;
    return {
        id,
        name,
        comment: migrateCommentJson(comment, typeDocJsonParserVersion),
        rest,
        optional,
        type: migrateTypeJson(type, typeDocJsonParserVersion)
    };
}
function migratePropertyJson(propertyJson, typeDocJsonParserVersion) {
    const [major, minor] = typeDocJsonParserVersion;
    const { id, name, comment, type } = propertyJson;
    if (major < 9) {
        if (major === 8 && minor < 2) {
            return {
                id,
                name,
                comment: migrateCommentJson(comment, typeDocJsonParserVersion),
                readonly: false,
                optional: false,
                type: migrateTypeJson(type, typeDocJsonParserVersion)
            };
        }
    }
    const { readonly, optional } = propertyJson;
    return {
        id,
        name,
        comment: migrateCommentJson(comment, typeDocJsonParserVersion),
        readonly,
        optional,
        type: migrateTypeJson(type, typeDocJsonParserVersion)
    };
}
function migrateSignatureJson(signatureJson, typeDocJsonParserVersion) {
    const [major] = typeDocJsonParserVersion;
    const { id, name, typeParameters, parameters, returnType } = signatureJson;
    if (major < 3) {
        return {
            id,
            name,
            comment: { description: null, blockTags: [], modifierTags: [] },
            typeParameters: typeParameters.map((typeParameterJson) => migrateTypeParameterJson(typeParameterJson, typeDocJsonParserVersion)),
            parameters: parameters.map((parameterJson) => migrateParameterJson(parameterJson, typeDocJsonParserVersion)),
            returnType: migrateTypeJson(returnType, typeDocJsonParserVersion)
        };
    }
    const { comment } = signatureJson;
    return {
        id,
        name,
        comment: migrateCommentJson(comment, typeDocJsonParserVersion),
        typeParameters: typeParameters.map((typeParameterJson) => migrateTypeParameterJson(typeParameterJson, typeDocJsonParserVersion)),
        parameters: parameters.map((parameterJson) => migrateParameterJson(parameterJson, typeDocJsonParserVersion)),
        returnType: migrateTypeJson(returnType, typeDocJsonParserVersion)
    };
}
function migrateSourceJson(sourceJson, typeDocJsonParserVersion) {
    const [major] = typeDocJsonParserVersion;
    const { line, file, path } = sourceJson;
    if (major < 3) {
        return { line, file, path, url: null };
    }
    const { url } = sourceJson;
    return { line, file, path, url };
}
function migrateTypeParameterJson(typeParameterJson, typeDocJsonParserVersion) {
    const [major] = typeDocJsonParserVersion;
    const { id, name, default: _default } = typeParameterJson;
    if (major < 7) {
        const { type } = typeParameterJson;
        return {
            id,
            name,
            constraint: type ? migrateTypeJson(type, typeDocJsonParserVersion) : null,
            default: _default ? migrateTypeJson(_default, typeDocJsonParserVersion) : null
        };
    }
    const { constraint } = typeParameterJson;
    return {
        id,
        name,
        constraint: constraint ? migrateTypeJson(constraint, typeDocJsonParserVersion) : null,
        default: _default ? migrateTypeJson(_default, typeDocJsonParserVersion) : null
    };
}
function migrateTypeJson(typeJson, typeDocJsonParserVersion) {
    const [major, minor] = typeDocJsonParserVersion;
    const { kind } = typeJson;
    switch (kind) {
        case TypeParser.Kind.Array: {
            const { type } = typeJson;
            return { kind, type: migrateTypeJson(type, typeDocJsonParserVersion) };
        }
        case TypeParser.Kind.Conditional: {
            const { checkType, extendsType, trueType, falseType } = typeJson;
            return {
                kind,
                checkType: migrateTypeJson(checkType, typeDocJsonParserVersion),
                extendsType: migrateTypeJson(extendsType, typeDocJsonParserVersion),
                trueType: migrateTypeJson(trueType, typeDocJsonParserVersion),
                falseType: migrateTypeJson(falseType, typeDocJsonParserVersion)
            };
        }
        case TypeParser.Kind.IndexedAccess: {
            const { objectType, indexType } = typeJson;
            return {
                kind,
                objectType: migrateTypeJson(objectType, typeDocJsonParserVersion),
                indexType: migrateTypeJson(indexType, typeDocJsonParserVersion)
            };
        }
        case TypeParser.Kind.Inferred: {
            const { type } = typeJson;
            return { kind, type };
        }
        case TypeParser.Kind.Intersection: {
            const { types } = typeJson;
            return { kind, types: types.map((type) => migrateTypeJson(type, typeDocJsonParserVersion)) };
        }
        case TypeParser.Kind.Intrinsic: {
            const { type } = typeJson;
            return { kind, type };
        }
        case TypeParser.Kind.Literal: {
            const { value } = typeJson;
            return { kind, value };
        }
        case TypeParser.Kind.Mapped: {
            const { parameter, parameterType, nameType, templateType, readonly, optional } = typeJson;
            return {
                kind,
                parameter,
                parameterType: migrateTypeJson(parameterType, typeDocJsonParserVersion),
                nameType: nameType ? migrateTypeJson(nameType, typeDocJsonParserVersion) : null,
                templateType: migrateTypeJson(templateType, typeDocJsonParserVersion),
                readonly,
                optional
            };
        }
        case TypeParser.Kind.NamedTupleMember: {
            const { name, type, optional } = typeJson;
            return { kind, name, type: migrateTypeJson(type, typeDocJsonParserVersion), optional };
        }
        case TypeParser.Kind.Optional: {
            const { type } = typeJson;
            return { kind, type: migrateTypeJson(type, typeDocJsonParserVersion) };
        }
        case TypeParser.Kind.Predicate: {
            const { name, asserts, type } = typeJson;
            return { kind, name, asserts, type: type ? migrateTypeJson(type, typeDocJsonParserVersion) : null };
        }
        case TypeParser.Kind.Query: {
            const { queryType } = typeJson;
            return { kind, queryType: migrateTypeJson(queryType, typeDocJsonParserVersion) };
        }
        case TypeParser.Kind.Reference: {
            const { id, name, packageName, typeArguments } = typeJson;
            return {
                kind,
                id,
                name,
                packageName,
                typeArguments: typeArguments.map((typeArgument) => migrateTypeJson(typeArgument, typeDocJsonParserVersion))
            };
        }
        case TypeParser.Kind.Reflection: {
            if (major < 8) {
                const { reflection } = typeJson;
                return {
                    kind,
                    properties: reflection?.children
                        ?.filter((child) => child.kind === ReflectionKind.Property)
                        .map((child) => PropertyParser.generateFromTypeDoc(child).toJSON()) ?? null,
                    signatures: reflection?.signatures?.map((signature) => SignatureParser.generateFromTypeDoc(signature).toJSON()) ?? null,
                    methods: reflection?.children
                        ?.filter((child) => child.kind === ReflectionKind.Method)
                        .map((child) => MethodParser.generateFromTypeDoc(child).toJSON()) ?? null
                };
            }
            if (major === 8 && minor === 0) {
                const { properties, signatures } = typeJson;
                return {
                    kind,
                    properties: properties?.map((property) => migratePropertyJson(property, typeDocJsonParserVersion)) ?? null,
                    signatures: signatures?.map((signature) => migrateSignatureJson(signature, typeDocJsonParserVersion)) ?? null
                };
            }
            const { properties, signatures, methods } = typeJson;
            return {
                kind,
                properties: properties?.map((property) => migratePropertyJson(property, typeDocJsonParserVersion)) ?? null,
                signatures: signatures?.map((signature) => migrateSignatureJson(signature, typeDocJsonParserVersion)) ?? null,
                methods: methods?.map((method) => migrateMethodJson(method, typeDocJsonParserVersion)) ?? null
            };
        }
        case TypeParser.Kind.Rest: {
            const { type } = typeJson;
            return { kind, type: migrateTypeJson(type, typeDocJsonParserVersion) };
        }
        case TypeParser.Kind.TemplateLiteral: {
            const { head, tail } = typeJson;
            return {
                kind,
                head,
                tail: tail.map(({ text, type }) => ({ type: migrateTypeJson(type, typeDocJsonParserVersion), text }))
            };
        }
        case TypeParser.Kind.Tuple: {
            const { types } = typeJson;
            return { kind, types: types.map((type) => migrateTypeJson(type, typeDocJsonParserVersion)) };
        }
        case TypeParser.Kind.TypeOperator: {
            const { operator, type } = typeJson;
            return { kind, operator, type: migrateTypeJson(type, typeDocJsonParserVersion) };
        }
        case TypeParser.Kind.Union: {
            const { types } = typeJson;
            return { kind, types: types.map((type) => migrateTypeJson(type, typeDocJsonParserVersion)) };
        }
        case TypeParser.Kind.Unknown: {
            const { name } = typeJson;
            return { kind, name };
        }
    }
}
function migrateFunctionJson(functionJson, namespaceParentId, typeDocJsonParserVersion) {
    const { id, name, comment, source, external, signatures } = functionJson;
    return {
        id,
        name,
        comment: migrateCommentJson(comment, typeDocJsonParserVersion),
        source: source ? migrateSourceJson(source, typeDocJsonParserVersion) : null,
        namespaceParentId,
        external,
        signatures: signatures.map((signatureJson) => migrateSignatureJson(signatureJson, typeDocJsonParserVersion))
    };
}
function migrateNamespaceJson(namespaceJson, namespaceParentId, typeDocJsonParserVersion) {
    const [major] = typeDocJsonParserVersion;
    const { id, name, comment, source, external, classes, enums, functions, interfaces, namespaces, typeAliases } = namespaceJson;
    if (major < 6) {
        const { constants } = namespaceJson;
        return {
            id,
            name,
            comment: migrateCommentJson(comment, typeDocJsonParserVersion),
            source: source ? migrateSourceJson(source, typeDocJsonParserVersion) : null,
            namespaceParentId,
            external,
            classes: classes.map((classJson) => migrateClassJson(classJson, id, typeDocJsonParserVersion)),
            enums: enums.map((enumJson) => migrateEnumJson(enumJson, id, typeDocJsonParserVersion)),
            functions: functions.map((functionJson) => migrateFunctionJson(functionJson, id, typeDocJsonParserVersion)),
            interfaces: interfaces.map((interfaceJson) => migrateInterfaceJson(interfaceJson, id, typeDocJsonParserVersion)),
            namespaces: namespaces.map((namespaceJson) => migrateNamespaceJson(namespaceJson, id, typeDocJsonParserVersion)),
            typeAliases: typeAliases.map((typeAliasJson) => migrateTypeAliasJson(typeAliasJson, id, typeDocJsonParserVersion)),
            variables: constants.map((constantJson) => migrateVariableJson(constantJson, id, typeDocJsonParserVersion))
        };
    }
    const { variables } = namespaceJson;
    return {
        id,
        name,
        comment: migrateCommentJson(comment, typeDocJsonParserVersion),
        source: source ? migrateSourceJson(source, typeDocJsonParserVersion) : null,
        namespaceParentId,
        external,
        classes: classes.map((classJson) => migrateClassJson(classJson, id, typeDocJsonParserVersion)),
        enums: enums.map((enumJson) => migrateEnumJson(enumJson, id, typeDocJsonParserVersion)),
        functions: functions.map((functionJson) => migrateFunctionJson(functionJson, id, typeDocJsonParserVersion)),
        interfaces: interfaces.map((interfaceJson) => migrateInterfaceJson(interfaceJson, id, typeDocJsonParserVersion)),
        namespaces: namespaces.map((namespaceJson) => migrateNamespaceJson(namespaceJson, id, typeDocJsonParserVersion)),
        typeAliases: typeAliases.map((typeAliasJson) => migrateTypeAliasJson(typeAliasJson, id, typeDocJsonParserVersion)),
        variables: variables.map((variableJson) => migrateVariableJson(variableJson, id, typeDocJsonParserVersion))
    };
}
function migrateTypeAliasJson(typeAliasJson, namespaceParentId, typeDocJsonParserVersion) {
    const { id, name, comment, source, external, typeParameters, type } = typeAliasJson;
    return {
        id,
        name,
        comment: migrateCommentJson(comment, typeDocJsonParserVersion),
        source: source ? migrateSourceJson(source, typeDocJsonParserVersion) : null,
        namespaceParentId,
        external,
        typeParameters: typeParameters.map((typeParameterJson) => migrateTypeParameterJson(typeParameterJson, typeDocJsonParserVersion)),
        type: migrateTypeJson(type, typeDocJsonParserVersion)
    };
}
function migrateVariableJson(variableJson, namespaceParentId, typeDocJsonParserVersion) {
    const { id, name, comment, source, external, type, value } = variableJson;
    return {
        id,
        name,
        comment: migrateCommentJson(comment, typeDocJsonParserVersion),
        source: source ? migrateSourceJson(source, typeDocJsonParserVersion) : null,
        namespaceParentId,
        external,
        type: migrateTypeJson(type, typeDocJsonParserVersion),
        value
    };
}
//# sourceMappingURL=migrateProjectJson.js.map